/*
The copyright in this software is being made available under the BSD License, included below. This software may be subject to other third party and contributor rights, including patent rights, and no such rights are granted under this license.
 
Copyright (c) 2013, ITU and ISO/IEC
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of the ITU and ISO/IEC nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "yuv_proc.h"
#include <stdlib.h>
#include <string.h>

#pragma warning( disable: 4996 )

enum YUVProcUsage
{
    YUV444ToRGB32File=0,
    RGB32ToYUV444File=1,
    FramePackingFile=2,
    FramePackWithBandSeparationFile=3,
    FramePackWithLiftedBandSeparationFile=4,
    FrameUnpackingFile=5,
    FrameUnpackWithBandSeparationFile=6,
    FrameUnpackWithLiftedBandSeparationFile=7,
    EvaluateDirectFramePacking=8,
    EvaluateBandSeparatedFramePacking=9,
    EvaluateLiftedBandSeparatedFramePacking=10,
    EvaluatePSNR=11,
    YUV444ToRGB24File=12,
    RGB24toYUV444File=13,
};



FileType eFileType;

int main( int argc, char *argv[] ) 
{
    int srcH, srcW, numBits,numFrames;
    char srcName[FILENAME_MAX];
    char desName[FILENAME_MAX];

    if ( argc != 9 ) 
    {
        printf( "yuv_proc usage input_file_name source_width source_height out_file_name file_type numBits numFrames\n" );
        printf( "Usage:\n" );
        printf( "0: YUV444ToRGB24\n" );
        printf( "1: RGB24ToYUV444\n" );
        printf( "2: FramePacking\n" );
        printf( "3: FramePackingWithBandSeparation\n" );
        printf( "4: FramePackingWithLiftedBandSeparation\n" );
        printf( "5: FrameUnpacking\n" );
        printf( "6: FrameUnpackingWithBandSeparation\n" );
        printf( "7: FrameUnpackingWithLiftedBandSeparation\n" );
        printf( "8: EvaluateDirectFramePacking\n" );
        printf( "9: EvaluateBandSeparatedFramePacking\n" );
        printf( "10: EvaluateLiftedBandSeparatedFramePacking\n" );
        printf( "11: EvaluatePSNR between between two YUVs\n" );
        printf("\n");
        printf("file_type: 0: RGB32, 1: YUV444. 2: RGB24 For frame packing, this argument specifies input file type. For frame unpacking, it specifies output file type.");
        return 0;
    }
    YUVProcUsage eUsage = (YUVProcUsage)( atoi( argv[1] ) );
    strcpy( srcName, argv[2] );
    srcW = atoi( argv[3] );
    srcH = atoi( argv[4] );
    strcpy( desName, argv[5] );
    eFileType = (FileType)atoi( argv[6] );
    numBits = atoi( argv[7] );
    numFrames = atoi( argv[8] );
    switch ( eUsage )
    {
    case YUV444ToRGB32File:     //0
    case RGB32ToYUV444File:     //1
        printf("\nNot supported");
        exit(0);
        break;
    case YUV444ToRGB24File:     //12
        ConvertYUV444ToRGB24File(srcName, desName, srcW, srcH, numFrames,numBits);
        break;
    case RGB24toYUV444File:     //12
        ConvertRGB24toYUV444File(srcName, desName, srcW, srcH, numFrames,numBits);
        break;
    case FramePackingFile:     //2
        if(numBits == 8)
            FramePacking( srcName, desName, srcH, srcW, 1, 1, eFileType, PERFORM_ANTI_ALIASING,DirectMode,numFrames);
        else if(numBits ==10)
    	    FramePacking10( srcName, desName, srcH, srcW, 1, 1, eFileType, PERFORM_ANTI_ALIASING,DirectMode,numFrames);
        else {
          printf( "Illegal input bit depth\n" );
          return 0;
        }
        break;
    case FramePackWithBandSeparationFile:     //3
        if(numBits == 8)
            FramePacking( srcName, desName, srcH, srcW, 1, 1, eFileType, PERFORM_ANTI_ALIASING, BandSeparationMode,numFrames);
	    else if(numBits ==10)
          FramePacking10( srcName, desName, srcH, srcW, 1, 1, eFileType, PERFORM_ANTI_ALIASING, BandSeparationMode,numFrames);
        else {
          printf( "Illegal input bit depth\n" );
          return 0;
        }
        break;
    case FramePackWithLiftedBandSeparationFile:     //4
        if(numBits == 8)
            FramePacking( srcName, desName, srcH, srcW, 1, 1, eFileType, PERFORM_ANTI_ALIASING, LiftedBandSeparationMode,numFrames);
        else if(numBits == 10)
          FramePacking10( srcName, desName, srcH, srcW, 1, 1, eFileType, PERFORM_ANTI_ALIASING, LiftedBandSeparationMode,numFrames);
        else {
          printf( "Illegal input bit depth\n" );
          return 0;
        }
        break;
    case FrameUnpackingFile:     //5
#if (DECODER_POST_PROCESSING)
        if(numBits == 8)
            FrameUnpacking( srcName, desName, srcH, srcW, 1, 1, eFileType, DirectMode ,numFrames);
        else if(numBits == 10)
	        FrameUnpacking10( srcName, desName, srcH, srcW, 1, 1, eFileType, DirectMode ,numFrames);
		else {
          printf( "Illegal input bit depth\n" );
          return 0;
        }
#else
        FrameUnpacking( srcName, desName, srcH, srcW, 0, 0, eFileType, DirectMode ,numFrames);
#endif
        break;
    case FrameUnpackWithBandSeparationFile:     //6
        if(numBits == 8)
            FrameUnpacking( srcName, desName, srcH, srcW, 0, 0, eFileType, BandSeparationMode,numFrames);
        else if(numBits == 10)
    		FrameUnpacking10( srcName, desName, srcH, srcW, 0, 0, eFileType, BandSeparationMode,numFrames);
		else {
          printf( "Illegal input bit depth\n" );
          return 0;
        }
        break;
    case FrameUnpackWithLiftedBandSeparationFile:     //7
        if(numBits == 8)
            FrameUnpacking( srcName, desName, srcH, srcW, 0, 0, eFileType, LiftedBandSeparationMode,numFrames);
        else if(numBits == 10)
            FrameUnpacking10( srcName, desName, srcH, srcW, 0, 0, eFileType, LiftedBandSeparationMode,numFrames);
		else {
          printf( "Illegal input bit depth\n" );
          return 0;
        }
        break;
    //---------------------------------
    //NOTE: Modes 8-10 are not working properly yet!!! Need to debug them
    //---------------------------------
    case EvaluateDirectFramePacking:     //8
    case EvaluateBandSeparatedFramePacking:     //9
    case EvaluateLiftedBandSeparatedFramePacking:     //10
        printf("\n Modes 8-10 Not fully functional yet!");
        break;
        //EvaluateFramePacking(srcName, srcH, srcW, 1, 1, eFileType, DirectMode);
        //EvaluateFramePacking(srcName, srcH, srcW, 0, 0, eFileType, BandSeparationMode);
        //EvaluateFramePacking(srcName, srcH, srcW, 0, 0, eFileType, LiftedBandSeparationMode);
        //break;
    case EvaluatePSNR:     //11
        if(numBits == 8)
            ComputePSNR(srcName, desName, srcW, srcH, eFileType,numFrames);
        else if(numBits == 10)
            ComputePSNR10(srcName, desName, srcW, srcH, eFileType,numFrames);
		else {
          printf( "Illegal input bit depth\n" );
          return 0;
        }
        break;
    default:
        break;
    }
}