/*
The copyright in this software is being made available under the BSD License, included below. This software may be subject to other third party and contributor rights, including patent rights, and no such rights are granted under this license.
 
Copyright (c) 2013, ITU and ISO/IEC
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of the ITU and ISO/IEC nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#ifndef YUV_PROC_H
#define YUV_PROC_H
#include <stdio.h>

typedef unsigned char byte;
typedef unsigned short read10;

//#define BT601
#define   PERFORM_ANTI_ALIASING     1       // value = (content_interpretation_type == 3 ? 0 : 1) 
                                            // Force for direct frame packing even in lossless compression if value = 1
#define   DECODER_POST_PROCESSING   1       // Force decoder post-processing, used in cases of huge quantization losses between main and aux frame

//#define DISPLAY_ALL_FRAMES_PSNR

enum FramePackingMode {
    DirectMode,
    BandSeparationMode,
    LiftedBandSeparationMode
};

enum YUVFormat {
    YUV_444,
    YUV_420,
    RGB_32,
    RGB_24,
    RGB_8
};

enum FileType
{
    RGB32,
    YUV444,
    YUV420
};

byte Round( double dIn );
void ColorConversion( byte R, byte G, byte B, byte* Y, byte* U, byte* V );
void InverseColorConversion( byte Y, byte U, byte V, byte* R, byte* G, byte* B );
void YUV444ToRGB32( char *InputName, char *OutName, int srcH, int srcW );
void RGB32ToYUV444( char *InputName, char *OutName, int srcH, int srcW );
void FramePacking( char *InputName, char *OutName, int srcH, int srcW, double dAlpha, double dBeta, FileType eFileType, bool bAntiAlias,FramePackingMode framePackingMode,int numFrames);
void FramePacking10( char *InputName, char *OutName, int srcH, int srcW, double dAlpha, double dBeta, FileType eFileType, bool bAntiAlias,FramePackingMode framePackingMode,int numFrames);
void FrameUnpacking( char *InputName, char *OutName, int srcH, int srcW, double dAlpha, double dBeta, FileType eFileType, FramePackingMode framePackingMode,int numFrames );
void FrameUnpacking10( char *InputName, char *OutName, int srcH, int srcW, double dAlpha, double dBeta, FileType eFileType, FramePackingMode framePackingMode,int numFrames );
void EvaluateFramePacking( char *InputName, int srcH, int srcW, double dAlpha, double dBeta, FileType eFileType, FramePackingMode framePackingMode,int numFrames);
void ComputePSNR( char *InputName, char *OutName, int srcW, int srcH, FileType eFileType,int numFrames);
void ComputePSNR10( char *InputName, char *OutName, int srcW, int srcH, FileType eFileType,int numFrames);

int ConvertYUV444ToRGB24File( char *InputName, char *OutName, int srcW, int srcH, int numFrames, int bitdepth);
int ConvertRGB24toYUV444File( char *InputName, char *OutName, int srcW, int srcH, int numFrames, int bitdepth);
#endif